@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -26,7 +26,7 @@
 #ifndef __EMPATHY_CONTACT_LIST_STORE_H__
 #define __EMPATHY_CONTACT_LIST_STORE_H__
 
-#include <gtk/gtktreestore.h>
+#include <gtk/gtk.h>
 
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact.h>
@@ -60,12 +60,15 @@ typedef enum {
 	EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE,
 	EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE,
 	EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR,
-	EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP,
+	EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+	EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+	EMPATHY_CONTACT_LIST_STORE_COL_FLAGS,
 	EMPATHY_CONTACT_LIST_STORE_COL_COUNT
 } EmpathyContactListStoreCol;
 
 struct _EmpathyContactListStore {
 	GtkTreeStore parent;
+	gpointer priv;
 };
 
 struct _EmpathyContactListStoreClass {
@@ -87,6 +90,9 @@ void                       empathy_contact_list_store_set_show_groups   (Empathy
 gboolean                   empathy_contact_list_store_get_is_compact     (EmpathyContactListStore     *store);
 void                       empathy_contact_list_store_set_is_compact     (EmpathyContactListStore     *store,
 									 gboolean                    is_compact);
+gboolean                   empathy_contact_list_store_get_show_protocols   (EmpathyContactListStore     *store);
+void                       empathy_contact_list_store_set_show_protocols   (EmpathyContactListStore     *store,
+									 gboolean                    show_protocols);
 EmpathyContactListStoreSort empathy_contact_list_store_get_sort_criterium (EmpathyContactListStore     *store);
 void                       empathy_contact_list_store_set_sort_criterium (EmpathyContactListStore     *store,
 									 EmpathyContactListStoreSort  sort_criterium);
@@ -101,6 +107,8 @@ gboolean                   empathy_contact_list_store_search_equal_func  (GtkTre
 									 const gchar                *key,
 									 GtkTreeIter                *iter,
 									 gpointer                    search_data);
+GdkPixbuf *                contact_list_store_get_contact_status_icon ( EmpathyContactListStore *store,
+									 EmpathyContact             *contact);
 
 G_END_DECLS
 
