@@ -115,15 +115,15 @@ account_assistant_build_error_page (EmpathyAccountAssistant *self,
   gtk_widget_show (w);
 
   if (page_num == PAGE_IMPORT)
-    message = _("There has been an error while importing the accounts.");
+    message = _("There was an error while importing the accounts.");
   else if (page_num >= PAGE_ENTER_CREATE &&
       priv->first_resp == RESPONSE_ENTER_ACCOUNT)
-    message = _("There has been an error while parsing the account details.");
+    message = _("There was an error while parsing the account details.");
   else if (page_num >= PAGE_ENTER_CREATE &&
       priv->first_resp == RESPONSE_CREATE_ACCOUNT)
-    message = _("There has been an error while creating the account.");
+    message = _("There was an error while creating the account.");
   else
-    message = _("There has been an error.");
+    message = _("There was an error.");
 
   w = gtk_label_new (message);
   gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -225,7 +225,7 @@ account_assistant_account_enabled_cb (GObject *source,
   EmpathyAccountAssistant *self = user_data;
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
 
-  empathy_account_set_enabled_finish (EMPATHY_ACCOUNT (source),
+  tp_account_set_enabled_finish (TP_ACCOUNT (source),
       result, &error);
 
   if (error)
@@ -249,10 +249,20 @@ account_assistant_apply_account_cb (GObject *source,
   EmpathyAccountAssistant *self = user_data;
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
-  EmpathyAccount *account;
+  TpAccount *account;
+  gchar *display_name;
 
   empathy_account_settings_apply_finish (settings, result, &error);
 
+  /* set default display name */
+  display_name = empathy_account_widget_get_default_display_name (
+      priv->current_widget_object);
+
+  empathy_account_settings_set_display_name_async (settings,
+      display_name, NULL, NULL);
+
+  g_free (display_name);
+
   priv->is_creating = FALSE;
 
   if (error != NULL)
@@ -265,7 +275,7 @@ account_assistant_apply_account_cb (GObject *source,
 
   /* enable the newly created account */
   account = empathy_account_settings_get_account (settings);
-  empathy_account_set_enabled_async (account, TRUE,
+  tp_account_set_enabled_async (account, TRUE,
       account_assistant_account_enabled_cb, self);
 }
 
@@ -930,8 +940,20 @@ empathy_account_assistant_init (EmpathyAccountAssistant *self)
 }
 
 GtkWidget *
-empathy_account_assistant_new (GtkWindow *window)
+empathy_account_assistant_show (GtkWindow *window)
 {
-  return g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT, "parent-window",
-      window, NULL);
+  static GtkWidget *dialog = NULL;
+
+  if (dialog == NULL)
+    {
+      dialog =  g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT, "parent-window",
+        window, NULL);
+      g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &dialog);
+    }
+
+  gtk_window_present (GTK_WINDOW (dialog));
+
+  return dialog;
 }
+
+
