@@ -38,6 +38,7 @@
 #include <libebook/e-book.h>
 #include <libnotify/notify.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/connection-manager.h>
@@ -49,7 +50,6 @@
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-account-settings.h>
 #include <libempathy/empathy-connectivity.h>
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-debugger.h>
 #include <libempathy/empathy-dispatcher.h>
@@ -81,6 +81,10 @@
 
 #define COMMAND_ACCOUNTS_DIALOG 1
 
+static gboolean account_dialog_only = FALSE;
+static gboolean start_hidden = FALSE;
+static gboolean no_connect = FALSE;
+
 static void
 dispatch_cb (EmpathyDispatcher *dispatcher,
     EmpathyDispatchOperation *operation,
@@ -100,33 +104,27 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
         (empathy_dispatch_operation_get_channel_wrapper (operation));
 
       id = empathy_tp_chat_get_id (tp_chat);
-      if (!id)
+      if (!EMP_STR_EMPTY (id))
         {
-          EmpathyContact *contact;
-
-          contact = empathy_tp_chat_get_remote_contact (tp_chat);
-          if (contact)
-            id = empathy_contact_get_id (contact);
-        }
-
-      if (id)
-        {
-          EmpathyAccountManager *manager;
           TpConnection *connection;
-          EmpathyAccount *account;
+          TpAccount *account;
 
-          manager = empathy_account_manager_dup_singleton ();
           connection = empathy_tp_chat_get_connection (tp_chat);
-          account = empathy_account_manager_get_account_for_connection (
-              manager, connection);
+          account = empathy_get_account_for_connection (connection);
           chat = empathy_chat_window_find_chat (account, id);
-          g_object_unref (manager);
         }
 
       if (chat)
-        empathy_chat_set_tp_chat (chat, tp_chat);
+        {
+          empathy_chat_set_tp_chat (chat, tp_chat);
+        }
       else
-        chat = empathy_chat_new (tp_chat);
+        {
+          chat = empathy_chat_new (tp_chat);
+          /* empathy_chat_new returns a floating reference as EmpathyChat is
+           * a GtkWidget. This reference will be taken by a container
+           * (a GtkNotebook) when we'll call empathy_chat_window_present_chat */
+        }
 
       empathy_chat_window_present_chat (chat);
 
@@ -154,11 +152,11 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
     }
 }
 
-/* Salut account creation */
+/* Salut account creation. The TpAccountManager first argument
+ * must already be prepared when calling this function. */
 static gboolean
-should_create_salut_account (void)
+should_create_salut_account (TpAccountManager *manager)
 {
-  EmpathyAccountManager *manager;
   gboolean salut_created = FALSE;
   GList *accounts, *l;
 
@@ -173,20 +171,20 @@ should_create_salut_account (void)
       return FALSE;
     }
 
-  manager = empathy_account_manager_dup_singleton ();
-  accounts = empathy_account_manager_dup_accounts (manager);
+  accounts = tp_account_manager_get_valid_accounts (manager);
 
   for (l = accounts; l != NULL;  l = g_list_next (l))
     {
-      EmpathyAccount *account = EMPATHY_ACCOUNT (l->data);
+      TpAccount *account = TP_ACCOUNT (l->data);
 
-      if (!tp_strdiff (empathy_account_get_protocol (account), "local-xmpp"))
-        salut_created = TRUE;
-
-      g_object_unref (account);
+      if (!tp_strdiff (tp_account_get_protocol (account), "local-xmpp"))
+        {
+          salut_created = TRUE;
+          break;
+        }
     }
 
-  g_object_unref (manager);
+  g_list_free (accounts);
 
   if (salut_created)
     {
@@ -205,7 +203,7 @@ salut_account_created (GObject *source,
     gpointer user_data)
 {
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
-  EmpathyAccount *account;
+  TpAccount *account;
   GError *error = NULL;
 
   if (!empathy_account_settings_apply_finish (settings, result, &error))
@@ -217,7 +215,7 @@ salut_account_created (GObject *source,
 
   account = empathy_account_settings_get_account (settings);
 
-  empathy_account_set_enabled_async (account, TRUE, NULL, NULL);
+  tp_account_set_enabled_async (account, TRUE, NULL, NULL);
   empathy_conf_set_bool (empathy_conf_get (),
       EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
       TRUE);
@@ -238,8 +236,12 @@ use_conn_notify_cb (EmpathyConf *conf,
 }
 
 static void
-create_salut_account_if_needed (EmpathyConnectionManagers *managers)
+create_salut_account_am_ready_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
 {
+  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+  EmpathyConnectionManagers *managers = user_data;
   EmpathyAccountSettings  *settings;
   TpConnectionManager *manager;
   const TpConnectionManagerProtocol *protocol;
@@ -252,22 +254,28 @@ create_salut_account_if_needed (EmpathyConnectionManagers *managers)
   gchar      *jid = NULL;
   GError     *error = NULL;
 
+  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
 
-  if (!should_create_salut_account ())
-    return;
+  if (!should_create_salut_account (account_manager))
+    goto out;
 
   manager = empathy_connection_managers_get_cm (managers, "salut");
   if (manager == NULL)
     {
       DEBUG ("Salut not installed, not making a salut account");
-      return;
+      goto out;
     }
 
   protocol = tp_connection_manager_get_protocol (manager, "local-xmpp");
   if (protocol == NULL)
     {
       DEBUG ("Salut doesn't support local-xmpp!!");
-      return;
+      goto out;
     }
 
   DEBUG ("Trying to add a salut account...");
@@ -278,7 +286,7 @@ create_salut_account_if_needed (EmpathyConnectionManagers *managers)
       DEBUG ("Failed to get self econtact: %s",
           error ? error->message : "No error given");
       g_clear_error (&error);
-      return;
+      goto out;
     }
 
   settings = empathy_account_settings_new ("salut", "local-xmpp",
@@ -320,20 +328,56 @@ create_salut_account_if_needed (EmpathyConnectionManagers *managers)
   g_object_unref (settings);
   g_object_unref (contact);
   g_object_unref (book);
+
+ out:
+  g_object_unref (managers);
+}
+
+static void
+create_salut_account_if_needed (EmpathyConnectionManagers *managers)
+{
+  TpAccountManager *manager;
+
+  manager = tp_account_manager_dup ();
+
+  tp_account_manager_prepare_async (manager, NULL,
+      create_salut_account_am_ready_cb, g_object_ref (managers));
+
+  g_object_unref (manager);
+}
+
+static gboolean
+has_non_salut_accounts (TpAccountManager *manager)
+{
+  gboolean ret = FALSE;
+  GList *accounts, *l;
+
+  accounts = tp_account_manager_get_valid_accounts (manager);
+
+  for (l = accounts ; l != NULL; l = g_list_next (l))
+    {
+      if (tp_strdiff (tp_account_get_protocol (l->data), "local-xmpp"))
+        {
+          ret = TRUE;
+          break;
+        }
+    }
+
+  g_list_free (accounts);
+
+  return ret;
 }
 
 static void
 maybe_show_account_assistant (void)
 {
-  EmpathyAccountManager *manager;
-  manager = empathy_account_manager_dup_singleton ();
+  TpAccountManager *manager;
+  manager = tp_account_manager_dup ();
 
-  if (empathy_account_manager_get_count (manager) == 0)
-    {
-      GtkWidget * assistant = empathy_account_assistant_new (
-        GTK_WINDOW (empathy_main_window_get ()));
-      gtk_window_present (GTK_WINDOW (assistant));
-    }
+  if (!has_non_salut_accounts (manager))
+    empathy_account_assistant_show (GTK_WINDOW (empathy_main_window_get ()));
+
+  g_object_unref (manager);
 }
 
 static gboolean
@@ -341,7 +385,7 @@ check_connection_managers_ready (EmpathyConnectionManagers *managers)
 {
   if (empathy_connection_managers_is_ready (managers))
     {
-      if (!empathy_import_mc4_accounts (managers))
+      if (!empathy_import_mc4_accounts (managers) && !start_hidden)
         maybe_show_account_assistant ();
 
       create_salut_account_if_needed (managers);
@@ -428,6 +472,58 @@ migrate_config_to_xdg_dir (void)
   g_free (old_dir);
 }
 
+static void
+do_show_accounts_ui (GtkWindow *window,
+    TpAccountManager *manager)
+{
+
+  GtkWidget *ui;
+
+  if (has_non_salut_accounts (manager))
+    ui = empathy_accounts_dialog_show (window, NULL);
+  else
+    ui = empathy_account_assistant_show (window);
+
+  if (account_dialog_only)
+    g_signal_connect (ui, "destroy",
+      G_CALLBACK (gtk_main_quit), NULL);
+}
+
+static void
+account_manager_ready_for_accounts_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+  GError *error = NULL;
+
+  if (!tp_account_manager_prepare_finish (manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  do_show_accounts_ui (user_data, manager);
+}
+
+static void
+show_accounts_ui (GtkWindow *window,
+    gboolean force)
+{
+  TpAccountManager *manager;
+
+  if (!force)
+    return;
+
+  manager = tp_account_manager_dup ();
+
+  tp_account_manager_prepare_async (manager, NULL,
+      account_manager_ready_for_accounts_cb, window);
+
+  g_object_unref (manager);
+}
+
 static UniqueResponse
 unique_app_message_cb (UniqueApp *unique_app,
     gint command,
@@ -435,17 +531,24 @@ unique_app_message_cb (UniqueApp *unique_app,
     guint timestamp,
     gpointer user_data)
 {
-  GtkWidget *window = user_data;
+  GtkWindow *window = user_data;
 
   DEBUG ("Other instance launched, presenting the main window. "
       "Command=%d, timestamp %u", command, timestamp);
 
   if (command == COMMAND_ACCOUNTS_DIALOG)
     {
-      empathy_accounts_dialog_show (GTK_WINDOW (window), NULL);
+      show_accounts_ui (NULL, TRUE);
     }
   else
     {
+      /* We're requested to show stuff again, disable the start hidden global
+       * in case the accounts wizard wants to pop up.
+       */
+      start_hidden = FALSE;
+
+      show_accounts_ui (window, FALSE);
+
       gtk_window_set_screen (GTK_WINDOW (window),
           unique_message_data_get_screen (data));
       gtk_window_set_startup_id (GTK_WINDOW (window),
@@ -534,14 +637,42 @@ default_log_handler (const gchar *log_domain,
 #endif /* ENABLE_DEBUG */
 
 static void
-account_manager_ready_cb (EmpathyAccountManager *manager,
-    GParamSpec *spec,
+account_manager_ready_cb (GObject *source_object,
+    GAsyncResult *result,
     gpointer user_data)
 {
-  if (!empathy_account_manager_is_ready (manager))
-    return;
+  TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+  GError *error = NULL;
+  EmpathyIdle *idle;
+  EmpathyConnectivity *connectivity;
+  gboolean autoconnect = TRUE;
+  TpConnectionPresenceType presence;
 
-  if (should_create_salut_account () || !empathy_import_mc4_has_imported ())
+  if (!tp_account_manager_prepare_finish (manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  /* Autoconnect */
+  idle = empathy_idle_dup_singleton ();
+  connectivity = empathy_connectivity_dup_singleton ();
+
+  presence = tp_account_manager_get_most_available_presence (manager, NULL,
+      NULL);
+
+  empathy_conf_get_bool (empathy_conf_get (),
+      EMPATHY_PREFS_AUTOCONNECT, &autoconnect);
+  if (autoconnect && !no_connect &&
+      tp_connection_presence_type_cmp_availability
+          (presence, TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
+            <= 0)
+      /* if current state is Offline, then put it online */
+      empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
+
+  if (should_create_salut_account (manager)
+      || !empathy_import_mc4_has_imported ())
     {
       EmpathyConnectionManagers *managers;
       managers = empathy_connection_managers_dup_singleton ();
@@ -552,10 +683,186 @@ account_manager_ready_cb (EmpathyAccountManager *manager,
             G_CALLBACK (connection_managers_ready_cb), NULL);
         }
     }
-  else
+  else if (!start_hidden)
     {
       maybe_show_account_assistant ();
     }
+
+  g_object_unref (idle);
+  g_object_unref (connectivity);
+}
+
+static EmpathyDispatcher *
+setup_dispatcher (void)
+{
+  EmpathyDispatcher *d;
+  GPtrArray *filters;
+  struct {
+    const gchar *channeltype;
+    TpHandleType handletype;
+  } types[] = {
+    /* Text channels with handle types none, contact and room */
+    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_NONE  },
+    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_CONTACT  },
+    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM  },
+    /* file transfer to contacts */
+    { TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, TP_HANDLE_TYPE_CONTACT  },
+    /* stream media to contacts */
+    { TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, TP_HANDLE_TYPE_CONTACT  },
+    /* stream tubes to contacts and rooms */
+    { TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, TP_HANDLE_TYPE_CONTACT  },
+    { TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, TP_HANDLE_TYPE_ROOM  },
+    /* d-bus tubes to contacts and rooms */
+    { TP_IFACE_CHANNEL_TYPE_DBUS_TUBE, TP_HANDLE_TYPE_CONTACT },
+    { TP_IFACE_CHANNEL_TYPE_DBUS_TUBE, TP_HANDLE_TYPE_ROOM  },
+    /* roomlists */
+    { TP_IFACE_CHANNEL_TYPE_ROOM_LIST, TP_HANDLE_TYPE_NONE },
+  };
+  gchar *capabilities[] = {
+    "org.freedesktop.Telepathy.Channel.Interface.MediaSignalling/ice-udp",
+    "org.freedesktop.Telepathy.Channel.Interface.MediaSignalling/gtalk-p2p",
+    NULL };
+  GHashTable *asv;
+  guint i;
+
+  /* Setup the basic Client.Handler that matches our client filter */
+  filters = g_ptr_array_new ();
+  asv = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING,
+           TP_IFACE_CHANNEL_TYPE_TEXT,
+        TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_INT,
+            TP_HANDLE_TYPE_CONTACT,
+        NULL);
+  g_ptr_array_add (filters, asv);
+
+  d = empathy_dispatcher_new (PACKAGE_NAME, filters, NULL);
+
+  g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
+  g_ptr_array_free (filters, TRUE);
+
+  /* Setup the an extended Client.Handler that matches everything we can do */
+  filters = g_ptr_array_new ();
+  for (i = 0 ; i < G_N_ELEMENTS (types); i++)
+    {
+      asv = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING, types[i].channeltype,
+        TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_INT, types[i].handletype,
+        NULL);
+
+      g_ptr_array_add (filters, asv);
+    }
+
+  asv = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType",
+          G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+        TP_IFACE_CHANNEL ".TargetHandleType",
+          G_TYPE_INT, TP_HANDLE_TYPE_CONTACT,
+        TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio",
+          G_TYPE_BOOLEAN, TRUE,
+        NULL);
+  g_ptr_array_add (filters, asv);
+
+  asv = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType",
+          G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+        TP_IFACE_CHANNEL ".TargetHandleType",
+          G_TYPE_INT, TP_HANDLE_TYPE_CONTACT,
+        TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo",
+          G_TYPE_BOOLEAN, TRUE,
+        NULL);
+  g_ptr_array_add (filters, asv);
+
+
+  empathy_dispatcher_add_handler (d, PACKAGE_NAME"MoreThanMeetsTheEye",
+    filters, capabilities);
+
+  g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
+  g_ptr_array_free (filters, TRUE);
+
+  return d;
+}
+
+static void
+account_status_changed_cb (TpAccount *account,
+    guint old_status,
+    guint new_status,
+    guint reason,
+    gchar *dbus_error_name,
+    GHashTable *details,
+    EmpathyChatroom *room)
+{
+  TpConnection *conn;
+
+  conn = tp_account_get_connection (account);
+
+  empathy_dispatcher_join_muc (conn,
+      empathy_chatroom_get_room (room), NULL, NULL);
+}
+
+static void
+account_manager_chatroom_ready_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+  EmpathyChatroomManager *chatroom_manager = user_data;
+  GList *accounts, *l;
+  GError *error = NULL;
+
+  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  accounts = tp_account_manager_get_valid_accounts (account_manager);
+
+  for (l = accounts; l != NULL; l = g_list_next (l))
+    {
+      TpAccount *account = TP_ACCOUNT (l->data);
+      TpConnection *conn;
+      GList *chatrooms, *p;
+
+      conn = tp_account_get_connection (account);
+
+      chatrooms = empathy_chatroom_manager_get_chatrooms (
+          chatroom_manager, account);
+
+      for (p = chatrooms; p != NULL; p = p->next)
+        {
+          EmpathyChatroom *room = EMPATHY_CHATROOM (p->data);
+
+          if (!empathy_chatroom_get_auto_connect (room))
+            continue;
+
+          if (conn == NULL)
+            {
+              g_signal_connect (G_OBJECT (account), "status-changed",
+                  G_CALLBACK (account_status_changed_cb), room);
+            }
+          else
+            {
+              empathy_dispatcher_join_muc (conn,
+                  empathy_chatroom_get_room (room), NULL, NULL);
+            }
+        }
+
+      g_list_free (chatrooms);
+    }
+
+  g_list_free (accounts);
+}
+
+static void
+chatroom_manager_ready_cb (EmpathyChatroomManager *chatroom_manager,
+    GParamSpec *pspec,
+    gpointer user_data)
+{
+  TpAccountManager *account_manager = user_data;
+
+  tp_account_manager_prepare_async (account_manager, NULL,
+      account_manager_chatroom_ready_cb, chatroom_manager);
 }
 
 int
@@ -566,7 +873,7 @@ main (int argc, char *argv[])
 #endif
   EmpathyStatusIcon *icon;
   EmpathyDispatcher *dispatcher;
-  EmpathyAccountManager *account_manager;
+  TpAccountManager *account_manager;
   EmpathyLogManager *log_manager;
   EmpathyChatroomManager *chatroom_manager;
   EmpathyCallFactory *call_factory;
@@ -574,25 +881,23 @@ main (int argc, char *argv[])
   GtkWidget *window;
   EmpathyIdle *idle;
   EmpathyConnectivity *connectivity;
-  gboolean autoconnect = TRUE;
-  gboolean no_connect = FALSE;
-  gboolean hide_contact_list = FALSE;
-  gboolean accounts_dialog = FALSE;
   GError *error = NULL;
   TpDBusDaemon *dbus_daemon;
   UniqueApp *unique_app;
+  gboolean chatroom_manager_ready;
 
+  GOptionContext *optcontext;
   GOptionEntry options[] = {
       { "no-connect", 'n',
         0, G_OPTION_ARG_NONE, &no_connect,
         N_("Don't connect on startup"),
         NULL },
-      { "hide-contact-list", 'h',
-        0, G_OPTION_ARG_NONE, &hide_contact_list,
-        N_("Don't show the contact list on startup"),
+      { "start-hidden", 'h',
+        0, G_OPTION_ARG_NONE, &start_hidden,
+        N_("Don't display the contact list or any other dialogs on startup"),
         NULL },
       { "accounts", 'a',
-        0, G_OPTION_ARG_NONE, &accounts_dialog,
+        0, G_OPTION_ARG_NONE, &account_dialog_only,
         N_("Show the accounts dialog"),
         NULL },
       { "version", 'v',
@@ -605,23 +910,27 @@ main (int argc, char *argv[])
   g_thread_init (NULL);
   empathy_init ();
 
-  if (!gtk_init_with_args (&argc, &argv, N_("- Empathy IM Client"),
-      options, GETTEXT_PACKAGE, &error))
-    {
-      g_warning ("Error in empathy init: %s", error->message);
-      return EXIT_FAILURE;
-    }
+  optcontext = g_option_context_new (N_("- Empathy IM Client"));
+  g_option_context_add_group (optcontext, gst_init_get_option_group ());
+  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
+#if HAVE_LIBCHAMPLAIN
+  g_option_context_add_group (optcontext, clutter_get_option_group ());
+#endif
+  g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
+
+  if (!g_option_context_parse (optcontext, &argc, &argv, &error)) {
+    g_print ("%s\nRun '%s --help' to see a full list of available command line options.\n",
+        error->message, argv[0]);
+    g_warning ("Error in empathy init: %s", error->message);
+    return EXIT_FAILURE;
+  }
+
+  g_option_context_free (optcontext);
 
   empathy_gtk_init ();
   g_set_application_name (_(PACKAGE_NAME));
   g_setenv ("PULSE_PROP_media.role", "phone", TRUE);
 
-  gst_init (&argc, &argv);
-
-#if HAVE_LIBCHAMPLAIN
-  gtk_clutter_init (&argc, &argv);
-#endif
-
   gtk_window_set_default_icon_name ("empathy");
   textdomain (GETTEXT_PACKAGE);
 
@@ -635,7 +944,7 @@ main (int argc, char *argv[])
 
   if (unique_app_is_running (unique_app))
     {
-      unique_app_send_message (unique_app, accounts_dialog ?
+      unique_app_send_message (unique_app, account_dialog_only ?
           COMMAND_ACCOUNTS_DIALOG : UNIQUE_ACTIVATE, NULL);
 
       g_object_unref (unique_app);
@@ -662,18 +971,19 @@ main (int argc, char *argv[])
       g_clear_error (&error);
     }
 
-  if (accounts_dialog)
+  if (account_dialog_only)
     {
-      GtkWidget *dialog;
-
-      dialog = empathy_accounts_dialog_show (NULL, NULL);
-      g_signal_connect (dialog, "destroy",
-                        G_CALLBACK (gtk_main_quit), NULL);
+      account_manager = tp_account_manager_dup ();
+      show_accounts_ui (NULL, TRUE);
 
       gtk_main ();
+
+      g_object_unref (account_manager);
       return 0;
     }
 
+  notify_init (_(PACKAGE_NAME));
+
   /* Setting up Idle */
   idle = empathy_idle_dup_singleton ();
   empathy_idle_set_auto_away (idle, TRUE);
@@ -685,33 +995,24 @@ main (int argc, char *argv[])
   empathy_conf_notify_add (empathy_conf_get (), EMPATHY_PREFS_USE_CONN,
       use_conn_notify_cb, connectivity);
 
-  /* Autoconnect */
-  empathy_conf_get_bool (empathy_conf_get (),
-      EMPATHY_PREFS_AUTOCONNECT, &autoconnect);
-  if (autoconnect && !no_connect &&
-      tp_connection_presence_type_cmp_availability
-          (empathy_idle_get_state (idle), TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
-            <= 0)
-      empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
-
   /* account management */
-  account_manager = empathy_account_manager_dup_singleton ();
-  g_signal_connect (account_manager, "notify::ready",
-      G_CALLBACK (account_manager_ready_cb), NULL);
+  account_manager = tp_account_manager_dup ();
+  tp_account_manager_prepare_async (account_manager, NULL,
+      account_manager_ready_cb, NULL);
+
+  /* Handle channels */
+  dispatcher = setup_dispatcher ();
+  g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
 
   migrate_config_to_xdg_dir ();
 
   /* Setting up UI */
   window = empathy_main_window_show ();
-  icon = empathy_status_icon_new (GTK_WINDOW (window), hide_contact_list);
+  icon = empathy_status_icon_new (GTK_WINDOW (window), start_hidden);
 
   g_signal_connect (unique_app, "message-received",
       G_CALLBACK (unique_app_message_cb), window);
 
-  /* Handle channels */
-  dispatcher = empathy_dispatcher_dup_singleton ();
-  g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
-
   /* Logging */
   log_manager = empathy_log_manager_dup_singleton ();
   empathy_log_manager_observe (log_manager, dispatcher);
@@ -719,7 +1020,17 @@ main (int argc, char *argv[])
   chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
   empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
 
-  notify_init (_(PACKAGE_NAME));
+  g_object_get (chatroom_manager, "ready", &chatroom_manager_ready, NULL);
+  if (!chatroom_manager_ready)
+    {
+      g_signal_connect (G_OBJECT (chatroom_manager), "notify::ready",
+          G_CALLBACK (chatroom_manager_ready_cb), account_manager);
+    }
+  else
+    {
+      chatroom_manager_ready_cb (chatroom_manager, NULL, account_manager);
+    }
+
   /* Create the call factory */
   call_factory = empathy_call_factory_initialise ();
   g_signal_connect (G_OBJECT (call_factory), "new-call-handler",
